@@ -7,7 +7,7 @@ require Exporter;
 
 our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw(get_news get_news_for_topic);
-our $VERSION   = '0.02';
+our $VERSION   = '0.05';
 
 use Carp;
 use LWP;
@@ -23,14 +23,19 @@ sub get_news {
   my $results = {};
   return unless $response->is_success;
 
+#  "&raquo;" >>
+#  my $re1 =  '<a name=(.*?)(?:<a name=|<br clear=all)';
+#  my $re2 =  '<a href="([^"]*)" id=r-\d-\d target=_blank><b>([^<]*)</b></a><br>';
   my $re1 =  '<td bgcolor=#efefef class=ks width=1% nowrap>(.*?)</td>';
   my $re2 =  '<a href="([^"]*)" id=r-\d-\d target=_blank><b>([^<]*)</b></a><br>'.
 	    '<font size=-1><font color=#6f6f6f><b>([^<]*)</font>'.
 	    '\s?<nobr>([^<]*)</nobr></b></font><br>'.
 	    '<font size=-1>([^<]*)<b>...</b>';
+#  my $re3 =  '<font size=-1>([^<]*)<b>...</b>';
 
   my @sections = split /($re1)/m,$response->content;
   my $current_section = '';
+#  print STDERR "total num is ".$#sections."\n";
   foreach my $section (@sections) {
     if ($section =~ m/$re1/m) {
       $current_section = $1;
@@ -44,20 +49,77 @@ sub get_news {
             $results->{$current_section} = [];
           }
           my $story_h = {};
-          $story_h->{url} = $1;
-          $story_h->{headline} = $2;
-	  $story_h->{source} = $3;
+	  my( $url, $headline, $source, $update_time, $summary ) = ( $1, $2, $3, $4, $5 );
+          $story_h->{url} = $url;
+          $story_h->{headline} = $headline;
+	  $story_h->{source} = $source;
 	  $story_h->{source} =~ s/&nbsp;-//g;
-	  $story_h->{update_time} = $4;
-	  $story_h->{summary} = $5;
+	  $story_h->{update_time} = $update_time;
+	  $story_h->{summary} = $summary;
           push(@{$results->{$current_section}},$story_h);
         }
       }
     }
   }
+  # print STDERR Dumper($results);
   return $results;
 }
 
+sub get_news_for_topic {
+
+    my $topic = uri_escape( $_[0] );
+
+    my @results = ();
+    my $url = "http://news.google.com.tw/news?hl=zh-TW&ned=ttw&q=$topic";
+    my $ua = LWP::UserAgent->new();
+    $ua->agent('Mozilla/5.0');
+
+    my $response = $ua->get($url);
+    return unless $response->is_success;
+
+#    print STDERR $url."\n";
+    my $re1 =  '<td bgcolor=#efefef class=ks width=1% nowrap>(.*?)&copy;2004 Google</font></center></body></html>';
+    my $re2 =  '<a href="([^"]*)" id=r-\d target=_blank>(.*?)</a><br>'.
+	'<font size=-1><font color=#6f6f6f>([^<]*)</font>'.
+	'\s?<nobr>([^<]*)</nobr></font><br>'.
+	'<font size=-1>(.*?)<b>...</b>';
+
+    my( $section ) = ( $response->content =~ m/$re1/s );
+    $section =~ s/\n//g;
+#    print STDERR $section."\n";
+    my @stories = split /($re2)/mi,$section;
+
+    foreach my $story (@stories) {
+	if ($story =~ m/$re2/i) {
+	    my $story_h = {};
+    
+	    my( $url, $headline, $source, $update_time, $summary ) = ( $1, $2, $3, $4, $5 );
+	    $source =~ s/&nbsp;/ /g;
+	    $source =~ s/\s+/ /g;
+	    $update_time =~ s/&nbsp;/ /g;
+	    $update_time =~ s/\s+/ /g;
+	    $update_time =~ s/-//g;
+	    $headline =~ s#<.+?>##gi;
+	    #$summary = $hs->parse($summary); $hs->eof;
+	    #$summary =~ s#<br># #gi;
+	    $summary =~ s#<.+?>##gi;
+
+	    $story_h->{url} = $url;
+	    $story_h->{headline} = $headline;
+	    $story_h->{source} = $source;
+	    $story_h->{update_time} = $update_time;
+	    #$story_h->{description} = "$source: $summary";
+	    $story_h->{summary} = $summary;
+
+	    push(@results,$story_h);
+
+	}
+    }
+
+    return \@results;
+
+}
+
 1;
 
 __END__
@@ -71,6 +133,8 @@ WWW::Google::News::TW - Access to Google's Taiwan News Service (Not Usenet)
   use WWW:Google::News::TW qw(get_news);
   my $results = get_news();
   
+  my $results = get_news_for_topic('金牌');
+
 =head1 DESCRIPTION
 
 This module provides a couple of methods to scrape results from Google Taiwan News, returning 
@@ -102,16 +166,21 @@ a data structure similar to the following (which happens to be suitable to feedi
 
 =item get_news()
 
-Scrapes L<http://news.google.com/news/gnmainlite.html> and returns a reference 
+Scrapes L<http://news.google.com.tw/news?ned=ttw> and returns a reference
 to a hash keyed on News Section, which points to an array of hashes keyed on URL and Headline.
 
+=item get_news_for_topic( $topic )
+
+Queries L<http://news.google.com.tw/news?ned=tw> for results on a particular topic, 
+and returns a pointer to an array of hashes containing result data. 
+
 =head1 SEE ALSO
 
 L<WWW::Google::News>, L<http://news.google.com.tw/>
 
 =head1 TODO
 
-* Add topic search
+* I haven't think about it yet....
 
 =head1 AUTHORS
 
